// Webpack
//
// Provides compilation and linting for Pug, TypeScript, JS, Stylus, Sass, CSS
// files, image minification and several file optimizations
//

// Dependencies
//
const webpack = require('webpack');
const babel = require('babel-polyfill');

const path = require('path');
const fs = require('fs-extra');
const poststylus = require('poststylus');

const environment = process.env.NODE_ENV || 'development';


// Babel Configuration
//
const babelConfig = {
  presets: [
    [ 'es2015', { loose: true, modules: false } ]
  ]
};

// Bundle entries used in individual layouts
//
let entries = {
  inkline: path.resolve(__dirname, '..', 'src', 'js', 'inkline.js'),
};


// Webpack Configuration
//
module.exports = {
  entry: entries,
  output: {
    path: path.resolve(__dirname, '..', 'dist', 'bundles'),
    filename: '[name].bundle.js'
  },

  devtool: 'source-map',

  module: {
    rules: [
      // JavaScript Linter
      {
        test: /\.js(x?)$/, enforce: 'pre',
        exclude: /(node_modules|vendors)/,
        loader: 'eslint-loader',
        options: require(path.resolve(__dirname, 'eslint.json'))
      },

      // JavaScript
      {
        test: /\.js(x?)$/,
        use: [
          { loader: 'uglify-loader' },
          { loader: 'babel-loader', options: babelConfig }
        ]
      },

      // Stylus Linter
      {
        test: /\.styl$/, enforce: 'pre',
        exclude: /(node_modules|vendors)/,
        loader: 'stylint-loader',
        options: {
          config: path.resolve(__dirname, 'stylint.json')
        }
      },

      // Stylus
      {
        test: /\.styl$/,
        use: [
          { loader: 'style-loader' },
          { loader: 'css-loader' },
          { loader: 'stylus-loader',
            options: {
              'resolve url': true,
              'include css': true,
              'include': 'node_modules',
              'url': 'embedurl',
              'use': [
                poststylus([
                  'autoprefixer',
                  'rucksack-css'
                ])
              ]
            }
          }
        ]
      },

      // CSS Linter
      {
        test: /\.css$/, enforce: 'pre',
        exclude: /(node_modules|vendors)/,
        loader: 'csslint-loader'
      },

      // CSS
      {
        test: /\.css$/,
        use: [
          { loader: 'style-loader' },
          { loader: 'css-loader' }
        ]
      },

      // JSON
      {
        test: /\.json$/,
        use: [
          { loader: 'json-loader' }
        ]
      },

      // Image
      {
        test: /\.(gif|png|jpe?g|svg)$/i,
        exclude: /(fonts)/,
        loaders: [
          {
            loader: 'file-loader',
            options: {
              context: path.join(__dirname, '..', 'src'),
              name: '[path][name].[ext]'
            }
          },
          { loader: 'image-webpack-loader',
            query: {
              progressive: true,
              optimizationLevel: 7,
              interlaced: false,

              gifsicle: { interlaced: false },
              optipng: { optimizationLevel: 7 },
              pngquant: { quality: '65-75', speed: 4 }
            }
          }
        ]
      },

      // Font
      {
        test: /\.(eot|svg|ttf|woff|woff2)$/i,
        exclude: /(img)/,
        loaders: [
          {
            loader: 'file-loader',
            options: {
              context: path.join(__dirname, '..', 'src'),
              name: '[path][name].[ext]'
            }
          }
        ]
      }

    ]
  },

  plugins: [
    new webpack.LoaderOptionsPlugin({
      options: {
        stylus: {
          use: [ require('poststylus')([ 'autoprefixer', 'rucksack-css' ]) ]
        }
      }
    }),
    new webpack.optimize.UglifyJsPlugin({
      minimize: true,
      debug: false
    }),
    new webpack.optimize.OccurrenceOrderPlugin()
  ]
    .concat(environment === 'development' ? [new webpack.HotModuleReplacementPlugin()] : []),

  resolve: {
    extensions: [ '.js', '.styl', '.css' ]
  }
};
